@@ -51,7 +51,14 @@ class WarRound(Enum):
         return self.name
 
 
-ELIXIR_TROOP_ORDER = [
+class Resource(Enum):
+    elixir = "Elixir"
+    builder_elixir = "Elixir2"
+    dark_elixir = "DarkElixir"
+    gold = "Gold"
+
+
+HOME_TROOP_ORDER = [
     "Barbarian",
     "Archer",
     "Giant",
@@ -67,9 +74,6 @@ ELIXIR_TROOP_ORDER = [
     "Electro Dragon",
     "Yeti",
     "Dragon Rider",
-]
-
-DARK_ELIXIR_TROOP_ORDER = [
     "Minion",
     "Hog Rider",
     "Valkyrie",
@@ -87,6 +91,7 @@ SIEGE_MACHINE_ORDER = [
     "Stone Slammer",
     "Siege Barracks",
     "Log Launcher",
+    "Flame Flinger"
 ]
 
 SUPER_TROOP_ORDER = [
@@ -97,16 +102,16 @@ SUPER_TROOP_ORDER = [
     "Super Wall Breaker",
     "Rocket Balloon",
     "Super Wizard",
+    "Super Dragon",
     "Inferno Dragon",
     "Super Minion",
     "Super Valkyrie",
     "Super Witch",
     "Ice Hound",
+    "Super Bowler"
 ]
 
-# TODO: When v2 comes around, remove siege machines from here
-HOME_TROOP_ORDER = ELIXIR_TROOP_ORDER + DARK_ELIXIR_TROOP_ORDER + \
-                   SIEGE_MACHINE_ORDER + SUPER_TROOP_ORDER
+HOME_TROOP_ORDER = HOME_TROOP_ORDER + SIEGE_MACHINE_ORDER
 
 
 BUILDER_TROOPS_ORDER = [
@@ -123,7 +128,7 @@ BUILDER_TROOPS_ORDER = [
     "Hog Glider",
 ]
 
-ELIXIR_SPELL_ORDER = [
+SPELL_ORDER = [
     "Lightning Spell",
     "Healing Spell",
     "Rage Spell",
@@ -131,9 +136,6 @@ ELIXIR_SPELL_ORDER = [
     "Freeze Spell",
     "Clone Spell",
     "Invisibility Spell",
-]
-
-DARK_ELIXIR_SPELL_ORDER = [
     "Poison Spell",
     "Earthquake Spell",
     "Haste Spell",
@@ -141,8 +143,6 @@ DARK_ELIXIR_SPELL_ORDER = [
     "Bat Spell",
 ]
 
-SPELL_ORDER = ELIXIR_SPELL_ORDER + DARK_ELIXIR_SPELL_ORDER
-
 HERO_ORDER = ["Barbarian King", "Archer Queen", "Grand Warden", "Royal Champion", "Battle Machine"]
 
 HERO_PETS_ORDER = ["L.A.S.S.I", "Electro Owl", "Mighty Yak", "Unicorn"]
